﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.Server.Core.InternalInterfaces;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Server.DataAccess;
using System.Threading;
using HeuristicLab.DataAccess.Interfaces;
using System.Transactions;
using HeuristicLab.Hive.Contracts;

namespace HeuristicLab.Hive.Server.Core {
  internal class DefaultScheduler : IScheduler {
    //private ISessionFactory factory;

    private static Mutex jobLock =
      new Mutex();

    #region IScheduler Members

    public DefaultScheduler() {
      //factory = ServiceLocator.GetSessionFactory();
    }

    public bool ExistsJobForSlave(HeuristicLab.Hive.Contracts.BusinessObjects.HeartBeatData hbData) {
      List<JobDto> allOfflineJobsForSlave =
        new List<JobDto>(DaoLocator.JobDao.FindFittingJobsForSlave(State.Offline, hbData.FreeCores, hbData.FreeMemory, hbData.SlaveId));
      return (allOfflineJobsForSlave != null && allOfflineJobsForSlave.Count > 0);
    }

    public HeuristicLab.Hive.Contracts.BusinessObjects.JobDto GetNextJobForSlave(Guid slaveId) {
      /// Critical section ///
      jobLock.WaitOne();
      JobDto jobToCalculate = null;
      using (TransactionScope scope = new TransactionScope(TransactionScopeOption.Required, new TransactionOptions { IsolationLevel = ApplicationConstants.ISOLATION_LEVEL_SCOPE })) {
        ClientDto slave = DaoLocator.ClientDao.FindById(slaveId);
        LinkedList<JobDto> allOfflineJobsForSlave =
          new LinkedList<JobDto>(DaoLocator.JobDao.FindFittingJobsForSlave(State.Offline, slave.NrOfFreeCores, slave.FreeMemory, slave.Id));        
        if (allOfflineJobsForSlave != null && allOfflineJobsForSlave.Count > 0) {
          jobToCalculate = allOfflineJobsForSlave.First.Value;
          jobToCalculate.State = State.Calculating;
          jobToCalculate.Client = slave;
          jobToCalculate.Client.State = State.Calculating;
          jobToCalculate.DateCalculated = DateTime.Now;
          DaoLocator.JobDao.AssignSlaveToJob(slave.Id, jobToCalculate.Id);
          DaoLocator.JobDao.Update(jobToCalculate);
          DaoLocator.ClientDao.Update(jobToCalculate.Client);
        }
        scope.Complete();
      }
      jobLock.ReleaseMutex();
      /// End Critical section ///

      return jobToCalculate;
    }

    #endregion
  }
}